--
--  Copyright (C) 2011 secunet Security Networks AG
--  Copyright (C) 2011 Reto Buerki <reet@codelabs.ch>
--  Copyright (C) 2011 Adrian-Ken Rueegsegger <ken@codelabs.ch>
--
--  This program is free software; you can redistribute it and/or modify it
--  under the terms of the GNU General Public License as published by the
--  Free Software Foundation; either version 2 of the License, or (at your
--  option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
--
--  This program is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--

with Ada.Real_Time;

with DHCP.Timer;
with DHCP.Logger;
with DHCP.Notify;
with DHCP.Timing_Events.Start;

with DHCPv4.States.Init;
with DHCPv4.Database;

package body DHCPv4.States.Requesting is

   package L renames DHCP.Logger;

   Instance : aliased Requesting_State_Type;

   -------------------------------------------------------------------------

   function Get_Name
     (State : not null access Requesting_State_Type)
      return String
   is
      pragma Unreferenced (State);
   begin
      return "Requesting";
   end Get_Name;

   -------------------------------------------------------------------------

   procedure Notify_Allocation (State : access Requesting_State_Type)
   is
      pragma Unreferenced (State);
   begin
      DHCP.Notify.Update (Reason => DHCP.Notify.Bound);
   end Notify_Allocation;

   -------------------------------------------------------------------------

   procedure Process_Nack
     (State       : access Requesting_State_Type;
      Transaction : in out Transaction_Type'Class;
      Msg         :        Message.Message_Type)
   is
      pragma Unreferenced (State, Msg);

      use Ada.Real_Time;

      Ev : DHCP.Timing_Events.Start.Start_Type (T => Transaction'Access);
   begin
      L.Log (Message => "Server rejected request, restarting in 10 seconds");

      --  Since we are about to start a fresh transaction all pending events
      --  are obsolete.

      DHCP.Timer.Clear;
      Database.Reset;
      Transaction.Reset;
      Transaction.Set_State (State => States.Init.State);

      Ev.Set_Time (At_Time => Clock + Seconds (S => 10));
      DHCP.Timer.Schedule (Event => Ev);
   end Process_Nack;

   -------------------------------------------------------------------------

   function State return DHCP.States.State_Handle
   is
   begin
      return Instance'Access;
   end State;

end DHCPv4.States.Requesting;
